package ants.controller;

import ants.model.Board;
import ants.gui.AntGUI;
import java.awt.Point;
import java.util.ArrayList;
import java.util.Random;

public class Controller {

    protected final int ANTS_COUNT;
    protected Random rand = new Random();

    protected Board board;
    protected AntGUI gui;
    protected final int NUM_THREADS = 4;

    public Controller(Board board, AntGUI gui) {
        this.board = board;
        this.gui = gui;
//        if (!board.getFlagLoaded()) {
            generateAnts(0.1);
//        }
        ANTS_COUNT = countAnts();
        guiThread();
        threadsStart();
        
    }

    private void generateAnts(double probability) {
        for (int i = 0; i < board.getWidth(); i++) {
            for (int j = 0; j < board.getHeight(); j++) {
                board.setValue(i, j, Math.random() < probability ? 1 : 0);
            }
        }
    }
    
    private int countAnts() {
        int count = 0;
        for (int i = 0; i < board.getWidth(); i++) {
            for (int j = 0; j < board.getHeight(); j++) {
                count++;
            }
        }
        return count;
    }

    private Point getRandomBoardCoordinates() {
        return new Point(rand.nextInt(board.getWidth()), rand.nextInt(board.getHeight()));
    }

    private Point randomShift(Point p, int maxShift) {
        Point shifted = new Point(p);
        shifted.x += rand.nextInt(maxShift * 2 + 1) - maxShift;
        shifted.y += rand.nextInt(maxShift * 2 + 1) - maxShift;
        return shifted;
    }

    public void guiStateChanged(boolean sync, boolean pick, boolean zeroone, int work, int dummy) {
        board.sync = sync;
        board.pickFirst = pick;
        board.allowZeroOne = zeroone;
        board.workerDelay = work;
        board.dummyOperations = dummy;
    }

    private void guiThread() {
        new Thread(() -> {
            long time = System.currentTimeMillis();
            while (true) {
                try {

                    Thread.sleep(50);
                    gui.setLabelText("" + (System.currentTimeMillis() - time));
                    int number = 0;

                    board.stop = true;
                    Thread.sleep(10);
                    for (int i = 0; i < board.getHeight(); i++) {
//                        Thread.sleep(10);
                        for (int j = 0; j < board.getWidth(); j++) {
                            number += board.getValue(j, i);
                        }
                    }
                    gui.repaint();

                    board.stop = false;
                    gui.setLabel2Text("" + number);
                    if (number != ANTS_COUNT) System.err.println("Counted: " + number);

                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }
        }).start();
    }

    private ArrayList<Controlthread> threads;

    private void threadsStart() {
        for (int i = 0; i < NUM_THREADS; i++) {
            Controlthread t = new Controlthread();
            t.start();
        }
    }

    private void randomSleep(int ms) {
        try {
            Thread.sleep(rand.nextInt(ms));
        } catch (InterruptedException ex) {
            ex.printStackTrace();
        }
    }

    private class Controlthread extends Thread {

        @Override
        public void run() {
            while (true) {

                Point from, to;
                
                do {
                    from = getRandomBoardCoordinates();
                } while (!board.pickAnt(from));
                
                do {
                    to = getRandomBoardCoordinates();
                } while (!board.putAnt(to));
                
//                do {
//                    from = getRandomBoardCoordinates();
//                    to = randomShift(from, 1);
//                } while (!board.moveAnt(from, to));
                
                randomSleep(board.workerDelay);
            }
        }
    }

}
