package ants.model;

import java.awt.Point;
import java.io.Serializable;

//public class Board implements Serializable{
public class Board {

    
    private int[][] board;
    
    private transient boolean loaded = false;
//    private transient volatile boolean stop = false;

    public boolean stop = false;

    // GUI controls
    public boolean sync = false;
    public boolean pickFirst = true;
    public boolean allowZeroOne = true;
    public int workerDelay = 10;
    public int dummyOperations = 0;


    public Board(int width, int height) {
        board = new int[width][height];
        stop = false;
        loaded = false;
    }
        
    public int getWidth() {
        return board.length;
    }

    public int getHeight() {
        return board[0].length;
    }

    public void setValue(int x, int y, int value) {
        board[x][y] = value;
    }

    public int getValue(int x, int y) {
        return board[x][y];
    }

    public boolean pickAnt(Point p) {
//    public synchronized boolean pickAnt(Point p) {
        if (!stop) {
            if (board[p.x][p.y] == 1) {
                board[p.x][p.y] = 0;
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    public boolean putAnt(Point p) {
//    public synchronized boolean putAnt(Point p) {
        if (p.x >= 0 && p.x < getWidth() && p.y >= 0 && p.y < getHeight()) {
            if (board[p.x][p.y] == 0) {
                board[p.x][p.y] = 1;
                return true;
            }
            return false;
        } else {
            return false;
        }
    }

    public boolean moveAnt(Point from, Point to) {
        
        if (sync) return moveAntSynchronized(from, to);
        else return moveAntUnsafe(from, to);   
    }
    
    public synchronized boolean moveAntSynchronized(Point from, Point to) {
        return move(from, to);
    }
    
    public boolean moveAntUnsafe(Point from, Point to) {
        return move(from, to);
    }
    
    public boolean move (Point from, Point to) {

        if (stop) return false;

        if (from.x < 0 || from.y < 0 || from.x >= getWidth() || from.y >= getHeight()) return false;
        if (to.x < 0 || to.y < 0 || to.x >= getWidth() || to.y >= getHeight()) return false;
        if (from.x == to.x && from.y == to.y) return false;
        
        if (board[from.x][from.y] <= 0 || board[to.x][to.y] > 0) return false;

        // Now we can move the ant!
        
        if (pickFirst) decrement(from);
        else increment(to);

        for (int i = 0; i < dummyOperations; i++) {}
        
        if (pickFirst) increment(to);
        else decrement(from);

        return true;
    }
    
    private void increment(Point p) {
        if (allowZeroOne) board[p.x][p.y] = 1;
        else board[p.x][p.y]++;
    }
    
    private void decrement(Point p) {
        if (allowZeroOne) board[p.x][p.y] = 0;
        else board[p.x][p.y]--;
    }

    public void setFlagLoaded() {
        loaded = true;
    }
    
    public boolean getFlagLoaded() {
        return loaded;
    }
}
